package Kevin::Plugin::kevin;
$Kevin::Plugin::kevin::VERSION = '0.4.0';
# ABSTRACT: DEPRECATED! Mojolicious plugin for alternative minion commands
use Mojo::Base 'Mojolicious::Plugin::Kevin::Commands';

1;

#pod =encoding utf8
#pod
#pod =head1 DESCRIPTION
#pod
#pod DEPRECATED! Use L<Mojolicious::Plugin::Kevin::Commands> instead.
#pod
#pod =head1 SEE ALSO
#pod
#pod L<Mojolicious::Plugin::Kevin::Commands>.
#pod
#pod =cut

__END__

=pod

=encoding UTF-8

=head1 NAME

Kevin::Plugin::kevin - DEPRECATED! Mojolicious plugin for alternative minion commands

=head1 VERSION

version 0.4.0

=head1 DESCRIPTION

DEPRECATED! Use L<Mojolicious::Plugin::Kevin::Commands> instead.

=head1 SEE ALSO

L<Mojolicious::Plugin::Kevin::Commands>.

=head1 AUTHOR

Adriano Ferreira <ferreira@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2017 by Adriano Ferreira.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
